package com.css.simulation.resource.scheduler.infra.db.mysql.mapper;

import api.common.pojo.po.scene.SceneComplexityPO;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.type.JdbcType;

import java.util.List;


@Mapper
public interface SceneComplexityMapper {

    @Insert("        insert into simulation.scene_complexity\n" +
            "        (complexity_id, scene_id, scene_type, rule_id,\n" +
            "         task_id, complexity, complexity_level,\n" +
            "         is_deleted, create_user_id, create_time\n" +
            "        )\n" +
            "        values (#{complexityId,jdbcType=VARCHAR}, #{sceneId,jdbcType=VARCHAR}, #{sceneType,jdbcType=VARCHAR},\n" +
            "                #{ruleId,jdbcType=VARCHAR},\n" +
            "                #{taskId,jdbcType=VARCHAR}, #{complexity,jdbcType=VARCHAR},\n" +
            "                #{complexityLevel,jdbcType=VARCHAR},\n" +
            "                #{isDeleted,jdbcType=VARCHAR},\n" +
            "                #{createUserId,jdbcType=VARCHAR}, #{createTime})")
    void saveSceneComplexity(SceneComplexityPO params);

    @Results(id = "complexityEvaluationResult", value = {
            @Result(column = "complexity_id", property = "complexityId", jdbcType = JdbcType.VARCHAR),
            @Result(column = "scene_id", property = "sceneId", jdbcType = JdbcType.VARCHAR),
            @Result(column = "scene_type", property = "sceneType", jdbcType = JdbcType.VARCHAR),
            @Result(column = "rule_id", property = "ruleId", jdbcType = JdbcType.VARCHAR),
            @Result(column = "rule_name", property = "ruleName", jdbcType = JdbcType.INTEGER),
            @Result(column = "task_id", property = "taskId", jdbcType = JdbcType.INTEGER),
            @Result(column = "complexity_level", property = "complexityLevel", jdbcType = JdbcType.INTEGER),
            @Result(column = "is_deleted", property = "isDeleted", jdbcType = JdbcType.INTEGER),
            @Result(column = "create_user_id", property = "createUserId", jdbcType = JdbcType.INTEGER),
            @Result(column = "create_time", property = "createTime", jdbcType = JdbcType.TIMESTAMP)
    })
    @Select("        SELECT " +
            "        *" +
            "        FROM scene_complexity " +
            "        WHERE " +
            "            is_deleted = '0' AND task_id = #{taskId} AND scene_id = #{sceneId}" +
            "        ORDER BY create_time DESC")
    List<SceneComplexityPO> selectSceneComplexityEvaluation(@Param("taskId") String taskId, @Param("sceneId") String sceneId);

    @Insert("<script>insert into simulation.scene_complexity" +
            "            (complexity_id, scene_id, scene_type, rule_id," +
            "    task_id, complexity, complexity_level," +
            "    is_deleted, create_user_id, create_time" +
            "    )" +
            "    values " +
            "     <foreach collection='list' item='item' separator=','>" +
            "            (#{item.complexityId,jdbcType=VARCHAR}, #{item.sceneId,jdbcType=VARCHAR}, #{item.sceneType,jdbcType=VARCHAR}," +
            "            #{item.ruleId,jdbcType=VARCHAR}," +
            "            #{item.taskId,jdbcType=VARCHAR}, #{item.complexity,jdbcType=VARCHAR}," +
            "            #{item.complexityLevel,jdbcType=VARCHAR}," +
            "            #{item.isDeleted,jdbcType=VARCHAR}," +
            "            #{item.createUserId,jdbcType=VARCHAR}, #{item.createTime})" +
            "    </foreach></script>")
    void saveSceneComplexityS(List<SceneComplexityPO> list);
}
